@@ -270,6 +270,16 @@ ephy_location_entry_get_property (GObject    *object,
   }
 }
 
+static void
+ephy_location_entry_constructed (GObject *object)
+{
+  G_OBJECT_CLASS (ephy_location_entry_parent_class)->constructed (object);
+
+#if GTK_CHECK_VERSION(3, 22, 20)
+  gtk_entry_set_input_hints (GTK_ENTRY (object), GTK_INPUT_HINT_NO_EMOJI);
+#endif
+}
+
 static void
 ephy_location_entry_finalize (GObject *object)
 {
@@ -368,6 +378,7 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 
   object_class->get_property = ephy_location_entry_get_property;
   object_class->set_property = ephy_location_entry_set_property;
+  object_class->constructed = ephy_location_entry_constructed;
   object_class->finalize = ephy_location_entry_finalize;
 
   widget_class->size_allocate = ephy_location_entry_size_allocate;
